import { AbstractControl, ControlValueAccessor, ValidationErrors, Validator } from '@angular/forms';
import { QueryOperatorDirective } from './query-operator.directive';
import { QueryFieldDirective } from './query-field.directive';
import { QueryEntityDirective } from './query-entity.directive';
import { QuerySwitchGroupDirective } from './query-switch-group.directive';
import { QueryButtonGroupDirective } from './query-button-group.directive';
import { QueryInputDirective } from './query-input.directive';
import { QueryRemoveButtonDirective } from './query-remove-button.directive';
import { ButtonGroupContext, Field, SwitchGroupContext, EntityContext, FieldContext, InputContext, LocalRuleMeta, OperatorContext, Option, QueryBuilderClassNames, QueryBuilderConfig, RemoveButtonContext, Rule, RuleSet } from './query-builder.interfaces';
import { ChangeDetectorRef, OnChanges, OnInit, QueryList, SimpleChanges, TemplateRef } from '@angular/core';
import { Entity } from '../index';
export declare const CONTROL_VALUE_ACCESSOR: any;
export declare const VALIDATOR: any;
export declare class QueryBuilderComponent implements OnInit, OnChanges, ControlValueAccessor, Validator {
    private changeDetectorRef;
    disabled: boolean;
    fields: Field[];
    filterFields: Field[];
    entities: Entity[];
    defaultClassNames: QueryBuilderClassNames;
    defaultOperatorMap: {
        [key: string]: string[];
    };
    data: RuleSet;
    onChangeCallback: () => void;
    onTouchedCallback: () => any;
    allowRuleset: boolean;
    classNames: QueryBuilderClassNames;
    operatorMap: {
        [key: string]: string[];
    };
    parentValue: RuleSet;
    config: QueryBuilderConfig;
    parentInputTemplates: QueryList<QueryInputDirective>;
    parentOperatorTemplate: QueryOperatorDirective;
    parentFieldTemplate: QueryFieldDirective;
    parentEntityTemplate: QueryEntityDirective;
    parentSwitchGroupTemplate: QuerySwitchGroupDirective;
    parentButtonGroupTemplate: QueryButtonGroupDirective;
    parentRemoveButtonTemplate: QueryRemoveButtonDirective;
    parentChangeCallback: () => void;
    parentTouchedCallback: () => void;
    buttonGroupTemplate: QueryButtonGroupDirective;
    switchGroupTemplate: QuerySwitchGroupDirective;
    fieldTemplate: QueryFieldDirective;
    entityTemplate: QueryEntityDirective;
    operatorTemplate: QueryOperatorDirective;
    removeButtonTemplate: QueryRemoveButtonDirective;
    inputTemplates: QueryList<QueryInputDirective>;
    private defaultTemplateTypes;
    private defaultEmptyList;
    private operatorsCache;
    private inputContextCache;
    private operatorContextCache;
    private fieldContextCache;
    private entityContextCache;
    private removeButtonContextCache;
    private buttonGroupContext;
    constructor(changeDetectorRef: ChangeDetectorRef);
    ngOnInit(): void;
    ngOnChanges(changes: SimpleChanges): void;
    validate(control: AbstractControl): ValidationErrors | null;
    value: RuleSet;
    writeValue(obj: any): void;
    registerOnChange(fn: any): void;
    registerOnTouched(fn: any): void;
    setDisabledState(isDisabled: boolean): void;
    findTemplateForRule(rule: Rule): TemplateRef<any>;
    findQueryInput(type: string): QueryInputDirective;
    getOperators(field: string): string[];
    getFields(entity: string): Field[];
    getInputType(field: string, operator: string): string;
    getOptions(field: string): Option[];
    getClassNames(...args: any[]): string;
    getDefaultField(entity: Entity): Field;
    getDefaultOperator(field: Field): string;
    addRule(parent?: RuleSet): void;
    removeRule(rule: Rule, parent?: RuleSet): void;
    addRuleSet(parent?: RuleSet): void;
    removeRuleSet(ruleset?: RuleSet, parent?: RuleSet): void;
    changeCondition(): void;
    changeOperator(): void;
    changeInput(): void;
    changeField(fieldValue: string, rule: Rule): void;
    changeEntity(entityValue: string, rule: Rule): void;
    getDefaultValue(defaultValue: any): any;
    getOperatorTemplate(): TemplateRef<any>;
    getFieldTemplate(): TemplateRef<any>;
    getEntityTemplate(): TemplateRef<any>;
    getButtonGroupTemplate(): TemplateRef<any>;
    getSwitchGroupTemplate(): TemplateRef<any>;
    getRemoveButtonTemplate(): TemplateRef<any>;
    getQueryItemClassName(local: LocalRuleMeta): string;
    getButtonGroupContext(): ButtonGroupContext;
    getRemoveButtonContext(rule: Rule): RemoveButtonContext;
    getFieldContext(rule: Rule): FieldContext;
    getEntityContext(rule: Rule): EntityContext;
    getSwitchGroupContext(): SwitchGroupContext;
    getOperatorContext(rule: Rule): OperatorContext;
    getInputContext(rule: Rule): InputContext;
    private checkEmptyRuleInRuleset;
    private validateRulesInRuleset;
    private handleDataChange;
    private handleTouched;
}
